// File mode flags

const MODE_RDONLY   = 0x01;
const MODE_WRONLY   = 0x02;
const MODE_RDWR     = 0x04;
const MODE_CREATE   = 0x08;
const MODE_APPEND   = 0x10;
const MODE_TRUNCATE = 0x20;
const MODE_SYNC     = 0x40;
const MODE_EXCL     = 0x80;

// Returns a nsIFile for the specified file in the profile chrome directory
function getUserChromeFile(fileName)
{
    var NSIFILE = Components.interfaces.nsIFile;
    var dirLocator = Components.classes["@mozilla.org/file/directory_service;1"]
                               .getService(Components.interfaces.nsIProperties);
    var userChromePath = dirLocator.get("UChrm", NSIFILE).path;

    var file = Components.classes["@mozilla.org/file/local;1"]
                         .createInstance(Components.interfaces.nsILocalFile);

    file.initWithPath(userChromePath);
    file.append(fileName);

    return file;
}

// Returns the contents of the specified file in the profile chrome directory
function readUserChromeFile(fileName) {
    var fileContents = "";
    var file = getUserChromeFile(fileName);

    if(file.exists()) {
        var ioFlags = MODE_RDONLY;

        // Get an input stream
        var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
                           .createInstance( Components.interfaces.nsIFileInputStream);
        is.init(file, ioFlags, 0, is.CLOSE_ON_EOF);
        var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
                            .createInstance( Components.interfaces.nsIScriptableInputStream);
        sis.init(is);

        // Read the file in
        while(sis.available() > 0)
            fileContents += sis.read(sis.available());

        // Close streams
        is.close();
        sis.close();
    }

    return fileContents;
}

// Writes the specified contents into the specified file in the profile chrome directory
function writeUserChromeFile(fileName, fileContents) {
    var file = getUserChromeFile(fileName);
    var ioFlags = MODE_WRONLY | MODE_CREATE | MODE_TRUNCATE;
    var perm = 0644;

    var os = Components.classes["@mozilla.org/network/file-output-stream;1"]
                       .createInstance( Components.interfaces.nsIFileOutputStream);
    os.init(file, ioFlags, perm, 0);

    var result = os.write(fileContents, fileContents.length);
    os.close();

    return result;
}

// Checks the if the user stylesheet already contains the import statement
function userStyleSheetHasImport() {
    var fileContents = readUserChromeFile('userContent.css');
    var re = new RegExp("^[ \t]*@import.*chrome://flashblock/content/flashblock.css", "m");

    return re.test(fileContents);
}

// Adds a CSS import statement for the flashblock stylesheet
function addImportToUserStylesheet(fileName) {
    var importStr = "@import url(chrome://flashblock/content/flashblock.css);"

    var fileContents = readUserChromeFile('userContent.css');
    var re = new RegExp("^[ \t]*@import.*chrome://flashblock/content/flashblock.css", "m");

    if(re.test(fileContents))
        return true;

    fileContents = importStr + "\n" + fileContents;

    var ret = writeUserChromeFile(fileName, fileContents);
    return (ret == fileContents.length);
}

// Removes the CSS import statement for the flashblock stylesheet
function removeImportFromUserStylesheet(fileName) {
    var fileContents = readUserChromeFile(fileName);
    var re = new RegExp("^[ \t]*@import.*chrome://flashblock/content/flashblock.css.*(\n)?$", "mg");

    if(re.test(fileContents)) {
        fileContents = fileContents.replace(re, '');
        var ret = writeUserChromeFile(fileName, fileContents);
        return (ret == fileContents.length)
    } else {
        return true;
    }
}

function onInstall() {
    window.removeEventListener("load", onInstall, true);

    if(! userStyleSheetHasImport()) {
        addImportToUserStylesheet('userContent.css')
        alert('Flashblock is now installed.\nPlease restart firefox to activate it.');
    }
}

window.addEventListener("load", onInstall, true);